#!/usr/bin/perl -w

# A script to read the fasta format and return the interleaved phylip format with AA color

use strict;
use warnings;


my $inFile      = "NuclearGene.fas";
my $lengthLimit = 90;         ### Sequence length in one line

my $outFile = $inFile.".html";

my $queryName = "Chimp";

my ($refGenetic_code, $front, $latter) = &variables;
my %genetic_code = %$refGenetic_code;


#################

$front =~ s/TITLE/$inFile/g;

open(IN,"$inFile");
my @inFileCont = <IN>;
close IN;


### Retrieve records from infile
my ($refFasHash, $refNameArray, $refSeqLength) = &fasHashReader(\@inFileCont);

#my $firstSeqName = $refNameArray->[0];

my $firstSeqName = "";
print "$queryName\n";
if($firstSeqName = join('', grep(/$queryName/, @$refNameArray))){
  print " FOUND\n";
} else {
  print "  NOT FOUND.\n";
  exit;
}

my %fashRec      = %$refFasHash;
my $firstSeq     = $fashRec{$firstSeqName};


### List the names in length order and pick up the longest name
##  The longest name will be used to determine the whitespace between name and sequence
my @nameLengths2D = ( );
my @sortedNames2D = ( );
foreach my $nameLine (@{$refNameArray}) {
  push(@nameLengths2D, [$nameLine,length($nameLine)]);
}
@sortedNames2D = sort{$b->[1] <=> $a->[1]}@nameLengths2D;
my $maxLength = $sortedNames2D[0][1];

## Printout
open(OUT,">$outFile");
print OUT "$front\n";
print OUT scalar@$refNameArray." $$refSeqLength\n\n";

my $lenCount = 1;
while($firstSeq){

  print OUT "$lenCount\n";
  $lenCount = $lenCount+$lengthLimit;
  my $refSeq = substr($firstSeq, 0, $lengthLimit);     ### Retrieve the first $lengthLimit characters
  $firstSeq =~ s/$refSeq//;       ### Delete the wrote sequence from $firstSeq.  $firstSeq is used in while.

  foreach my $name (@$refNameArray) {
    my $part = substr($refFasHash->{$name}, 0, $lengthLimit);     ### Retrieve the first $lengthLimit characters
    my $name2 = $name;
    $name2 =~ s/^>//g;

    ### Change color

    my $part2 = "";
    for(my $i=0; $i < (length($part) - 2) ; $i += 3) {
      my $refSeqCodon = substr($refSeq, $i,3);
      my $codon       = substr($part,   $i,3);
      if($name eq "$firstSeqName"){
        $part2 .= DNA2AA_Nuc($name, $codon, $refSeqCodon);
      } else {
        $part2 .= DNA2AA_Nuc_matchFirst($name, $codon, $refSeqCodon);
      }
    }

    ### Printout
    my $whiteString = " " x ($maxLength - length($name));
    if($name2 eq $queryName){
      print OUT "<u>$name2$whiteString $part2</u>\n";
    } else {
      print OUT "$name2$whiteString $part2\n";
    }
    $refFasHash->{$name} =~ s/$part//;
  }

  print OUT "\n";

  $refSeq = "";
}
print OUT "$latter\n";
close OUT;

exit;



######################################################################
####### Subroutine

sub fasHashReader{

  my $uniqNameSR = "";
  my $seqSR = "";
  my $seqSRLength = "";
  my $flag = 0;
  my @nameArraySR = ( );
  my %fasHashSR = ( );

  my ($inFileContSR) = @_;

  foreach my $inFileLine (@$inFileContSR) {
    chomp($inFileLine);
    if($inFileLine =~ /^>/){  
      if($flag == 0){$uniqNameSR = $inFileLine;
        $flag = 1;  
      } else {
        $fasHashSR{$uniqNameSR} = $seqSR;  
        push(@nameArraySR,$uniqNameSR);
        $uniqNameSR = "$inFileLine";
        $seqSR = "";
      }
    } else {
      $seqSR .= $inFileLine; 
    }
  }
  $fasHashSR{$uniqNameSR} = $seqSR;
  $seqSRLength = length($seqSR);
  push(@nameArraySR,$uniqNameSR);

  return (\%fasHashSR,\@nameArraySR,\$seqSRLength);

}



#####
sub DNA2AA_Nuc {

    my($SpName,$tri,$triHQS) = @_;
    
    $tri = uc $tri;   # Uppercase.

    if(exists $genetic_code{$tri}) {
      my $colCodon = $genetic_code{$tri};
      $colCodon =~ s/CODON/$tri/;
      return $colCodon;
    }
    else{
      return "$tri";
    }
}


sub DNA2AA_Nuc_matchFirst {

    my($SpName,$tri,$triHQS) = @_;
    
    $tri = uc $tri;   # Uppercase.
    
    if(exists $genetic_code{$tri}) {
      my $matchFirstTri = "";
      for(my $b=0 ;$b<3 ;$b++){
      
        ## Making match first
        my $baseHQS = substr($triHQS,$b,1);
        my $base    = substr($tri,   $b,1);
        my $base2   = "";
        if ($base eq $baseHQS){
          $base2   = "." 
        } else {
          $base2   = "$base"         
        }
        #print "  $b $base $baseHQS $base2\n";
        $matchFirstTri .= $base2;
      }
      
      my $colCodon = $genetic_code{$tri};
      $colCodon =~ s/CODON/$matchFirstTri/;
      #print "colCodon $colCodon\n";
      return $colCodon;
    }

    else{
      return "$tri";
    }

}


sub variables {

my(%genetic_code) = (

'TCA' => '<span class=ser>CODON</span>',    #'S',    # ser
'TCC' => '<span class=ser>CODON</span>',    #'S',    # ser
'TCG' => '<span class=ser>CODON</span>',    #'S',    # ser
'TCT' => '<span class=ser>CODON</span>',    #'S',    # ser
'TTC' => '<span class=phe>CODON</span>',    # F
'TTT' => '<span class=phe>CODON</span>',    #'F',    # phe
'TTA' => '<span class=leu>CODON</span>',    #'L',    # leu
'TTG' => '<span class=leu>CODON</span>',    #'L',    # leu
'TAC' => '<span class=tyr>CODON</span>',    #'Y',    # tyr
'TAT' => '<span class=tyr>CODON</span>',    #'Y',    # tyr
'TAA' => '<span class=stp>CODON</span>',    #'*',    # stp
'TAG' => '<span class=stp>CODON</span>',    #'*',    # stp
'TGC' => '<span class=cys>CODON</span>',    #'C',    # cys
'TGT' => '<span class=cys>CODON</span>',    #'C',    # cys
'TGA' => '<span class=stp>CODON</span>',    #'*',    # stp
'TGG' => '<span class=trp>CODON</span>',    #'W',    # trp
'CTA' => '<span class=leu>CODON</span>',    #'L',    # leu
'CTC' => '<span class=leu>CODON</span>',    #'L',    # leu
'CTG' => '<span class=leu>CODON</span>',    #'L',    # leu
'CTT' => '<span class=leu>CODON</span>',    #'L',    # leu
'CCA' => '<span class=pro>CODON</span>',    #'P',    # pro
'CCC' => '<span class=pro>CODON</span>',    #'P',    # pro
'CCG' => '<span class=pro>CODON</span>',    #'P',    # pro
'CCT' => '<span class=pro>CODON</span>',    #'P',    # pro
'CAC' => '<span class=his>CODON</span>',    #'H',    # his
'CAT' => '<span class=his>CODON</span>',    #'H',    # his
'CAA' => '<span class=gln>CODON</span>',    #'Q',    # gln
'CAG' => '<span class=gln>CODON</span>',    #'Q',    # gln
'CGA' => '<span class=arg>CODON</span>',    #'R',    # arg
'CGC' => '<span class=arg>CODON</span>',    #'R',    # arg
'CGG' => '<span class=arg>CODON</span>',    #'R',    # arg
'CGT' => '<span class=arg>CODON</span>',    #'R',    # arg
'ATA' => '<span class=ile>CODON</span>',    #'I',    # ile
'ATC' => '<span class=ile>CODON</span>',    #'I',    # ile
'ATT' => '<span class=ile>CODON</span>',    #'I',    # ile
'ATG' => '<span class=met>CODON</span>',    #'M',    # met
'ACA' => '<span class=thr>CODON</span>',    #'T',    # thr
'ACC' => '<span class=thr>CODON</span>',    #'T',    # thr
'ACG' => '<span class=thr>CODON</span>',    #'T',    # thr
'ACT' => '<span class=thr>CODON</span>',    #'T',    # thr
'AAC' => '<span class=asn>CODON</span>',    #'N',    # asn
'AAT' => '<span class=asn>CODON</span>',    #'N',    # asn
'AAA' => '<span class=lys>CODON</span>',    #'K',    # lys
'AAG' => '<span class=lys>CODON</span>',    #'K',    # lys
'AGC' => '<span class=ser>CODON</span>',    #'S',    # ser
'AGT' => '<span class=ser>CODON</span>',    #'S',    # ser
'AGA' => '<span class=arg>CODON</span>',    #'R',    # arg
'AGG' => '<span class=arg>CODON</span>',    #'R',    # arg
'GTA' => '<span class=val>CODON</span>',    #'V',    # val
'GTC' => '<span class=val>CODON</span>',    #'V',    # val
'GTG' => '<span class=val>CODON</span>',    #'V',    # val
'GTT' => '<span class=val>CODON</span>',    #'V',    # val
'GCA' => '<span class=ala>CODON</span>',    #'A',    # ala
'GCC' => '<span class=ala>CODON</span>',    #'A',    # ala
'GCG' => '<span class=ala>CODON</span>',    #'A',    # ala
'GCT' => '<span class=ala>CODON</span>',    #'A',    # ala
'GAC' => '<span class=asp>CODON</span>',    #'D',    # asp
'GAT' => '<span class=asp>CODON</span>',    #'D',    # asp
'GAA' => '<span class=glu>CODON</span>',    #'E',    # glu
'GAG' => '<span class=glu>CODON</span>',    #'E',    # glu
'GGA' => '<span class=gly>CODON</span>',    #'G',    # gly
'GGC' => '<span class=gly>CODON</span>',    #'G',    # gly
'GGG' => '<span class=gly>CODON</span>',    #'G',    # gly
'GGT' => '<span class=gly>CODON</span>',    #'G',    # gly
);

my $front = 
'
<!DOCTYPE html>
<html><head>
    <meta http-equiv="Content-Type" content="text/html">
    <title>TITLE</title>
    <style type="text/css">
    .ala { background-color: #ff00ff; }
    .arg { background-color: #ff0080; }
    .asn { background-color: #0080ff; }
    .asp { background-color: #00ffff; }
    .cys { background-color: #80ff00; }
    .gln { background-color: #ffff00; }
    .glu { background-color: #ffffdf; }
    .gly { background-color: #7f7f7f; }
    .his { background-color: #ff80ff; }
    .ile { background-color: #ff80c0; }
    .leu { background-color: #80bfff; }
    .lys { background-color: #80ffff; }
    .met { background-color: #80ff80; }
    .phe { background-color: #ffff80; }
    .pro { background-color: #ff8080; }
    .ser { background-color: #bfbfbf; }
    .thr { background-color: #ffbfff; }
    .tyr { background-color: #ffbfdf; }
    .trp { background-color: #bfdfff; }
    .val { background-color: #bfffff; }
    .stp { background-color: #ff0000; color: white}
    </style>
  </head>

  <body>
	<pre>
  ';

  my $latter = 
  '
  	  </pre>
    </body>
  </html>
  ';

  return(\%genetic_code, $front, $latter);

}